#!/usr/local/bin/perl
#######################################################
#
# Dmiry <Legh> Shulgachik
# http://legh.ru/jx     legh@legh.ru
#
# The software is provided "as is", without warranty of any kind,
# express or implied, including but not limited to the warranties
# of merchantability, fitness for a particular purpose, and
# noninfringement. In no event shall the author(s) be liable for any
# claim, damages, or other liability, whether in an action of
# contract, tort, or otherwise, arising from, out of, or in connection
# with the software or the use or other dealings in the software.
#
# SUPPORT
# Send bug reports to: <legh@legh.ru>
#
# LICENSE
# This software is in the public domain.
#
# v1.00      15.12.2008
# v1.23      22.04.2009
#
use constant script_version => 'v1.23';
#
#  useage
#      jx  [-j] [-l] [-h] jid|jhostname|jname [command]
#
#######################################################

use strict;
use warnings;

use constant use_config => 0;

my %config =(
	command          => 'bash',
	use_jid          => 0,
	show_tips        => 1,
);

my $jid=undef;
my %list_name2hostname;
my %list_name2nice;
my %list_hostname2name;
my @nlist;
my @cmd;
my %jls;

my $jexec_command=' /usr/sbin/jexec ';

if ( use_config )
{
	my @configpath = qw( /usr/local/etc/ /etc/ . );
	eval {
		require Config::General;
		%config = Config::General::ParseConfig(-DefaultConfig => \%config,-ConfigFile => 'jx.cfg',
							-AutoTrue=>'true',
							-MergeDuplicateOptions=> 'yes',
							-ConfigPath => \@configpath );
	} or exit_use_config();
}

sub basename {
	my @path=split('/',$0);
	my @rpath;
	foreach (@path){
		if ( $_ ne '' ) { push(@rpath,$_); }
	}
	return $rpath[$#rpath];
}

sub exit_use_config {
	print basename().": Can't load configuration file :(\n";
	exit(2);
}

sub exit_not_exec($) {
	print basename().": Can't jexec - no such jail: ".(shift @_)." :(\n";
	exit(1);
}

sub exit_jail_is_stop($) {
	print basename().": Can't jexec - jail is stop: ".(shift @_)." :(\n";
	exit(5);
}

sub exit_jail_unexist($){
	if ($_[0]<0){
		print basename().": Can't jexec - jail rcjid can't be less then 0: ".(shift @_)." :(\n";
	}else{
		print basename().": Can't jexec - jail : ".(shift @_)." : is unexist, max rcjid : ".($#nlist)." :(\n";
	}
	exit(6);
}

sub version{
	print basename().' is a wrapper to "jexec", '.script_version."\n";
}

sub help {
	version();
	print "\tUsage: ".basename()."  [-j] [-h] [-l] jid|jhostname|jname [command]\n";
}

sub fill_list {
	my @rc;
	open FF, '</etc/rc.conf';
	@rc=<FF>;
	close FF;

	foreach (@rc) {
		if ( $_ =~ m/[\ \t]*jail\_([a-zA-Z0-9\ \-\_\.]*)\_hostname[\ \t]*=[\ \t]*[\'\"]?([a-zA-Z0-9\ \.\-\_]*)[\'\"]?[\ \t]?/g )
		{
			$list_name2hostname{$1}=$2;
			$list_hostname2name{$2}=$1;
			push(@nlist,$2);
		} elsif ( $_ =~ m/[\ \t]*jail\_([a-zA-Z0-9\ \-\_\.]*)\_nice[\ \t]*=[\ \t]*[\'\"]?(\d*)[\'\"]?[\ \t]?/g )
		{
			$list_name2nice{$1}=$2;
		}
	}
}

sub renice_name {
	my $name=shift;
	if ( defined($list_name2nice{$name}) )
	{
		$jexec_command="nice -n $list_name2nice{$name} ".$jexec_command;
	}
}

sub renice_rcjid {
	my $rcjid=shift;
	if ( defined($list_name2nice{$list_hostname2name{$nlist[$rcjid]}}) )
	{
		$jexec_command="nice -n $list_name2nice{$list_hostname2name{$nlist[$rcjid]}} ".$jexec_command;
	}
}

sub renice_jid {
	my $jid=shift;
	if ( defined($list_hostname2name{$jls{$jid}}) and defined($list_name2nice{$list_hostname2name{$jls{$jid}}}) )
	{
		$jexec_command="nice -n $list_name2nice{$list_hostname2name{$jls{$jid}}} ".$jexec_command;
	}
}

sub renice_hostname {
	my $jhostname=shift;
	if ( defined($list_name2nice{$list_hostname2name{$jhostname}}) )
	{
		$jexec_command="nice -n $list_name2nice{$list_hostname2name{$jhostname}} ".$jexec_command;
	}
}

sub showlist {
	if ( $config{use_jid} ){
		exec '/usr/sbin/jls';
		exit;
	}
	fill_list();

	my $max_length_hostname=15;
	for (my $i=0;$i<$#nlist+1;$i++){
		if (length($nlist[$i]) > $max_length_hostname) {
			$max_length_hostname=length($nlist[$i]);
		}
	}

	if ( scalar(keys %list_name2nice)>0 )
	{
		printf ("\t%2s %2s\t%-".$max_length_hostname."s\t%-15s\n",'JID','Ni','Hostname','RC Name');
		for (my $i=0;$i<$#nlist+1;$i++){
			my $nice= defined($list_name2nice{$list_hostname2name{$nlist[$i]}})?
				$list_name2nice{$list_hostname2name{$nlist[$i]}} : '';
			printf ("\t%2d %2s\t%-".$max_length_hostname."s\t%-15s\n",$i,$nice,$nlist[$i],$list_hostname2name{$nlist[$i]});
		}
	}else
	{
		printf ("\t%2s\t%-".$max_length_hostname."s\t%-15s\n",'JID','Hostname','RC Name');
		for (my $i=0;$i<$#nlist+1;$i++){
			my $nice= defined($list_name2nice{$list_hostname2name{$nlist[$i]}})?
				$list_name2nice{$list_hostname2name{$nlist[$i]}} : '';
			printf ("\t%2d\t%-".$max_length_hostname."s\t%-15s\n",$i,$nlist[$i],$list_hostname2name{$nlist[$i]});
		}
	}
	exit;
}

if ( $#ARGV < 0 ){
	help();
	exit(4);
}

#read options
my $cmdbegin=1;
my $showlist=0;
while ( defined (my $option=shift @ARGV) )
{
	if ( $cmdbegin ) {
		if ( $option =~ m/^(-j|--jid)$/ )
		{
			$config{use_jid}=$config{use_jid}? 0 : 1 ;
		}
		elsif ( $option =~ m/^(-l|--list|l)$/ )
		{
			$showlist=1;
		}
		elsif ( $option =~ m/^(-h|--help)$/ )
		{
			help();
			exit;
		}
		elsif ( $option =~ m/^(-v|--version)$/ )
		{
			version();
			exit;
		}
		else
		{
			showlist() if $showlist;
			$cmdbegin=0;
			push(@cmd,$option);
		}
	} else {
		push(@cmd,$option);
	}
}
showlist() if $showlist;

# if only -j options
if ( $#cmd<0 ) {
	print basename().": Can't jexec - no jail is given :(\n";
	exit(4);
}

# if no command
if ( $#cmd==0 ){
	push(@cmd,$config{command});
}

if ( $cmd[0]=~ m/^(\-?)(\d+)$/ ) {
	$jid=$2;
	if (defined($1) and length($1)>0){
		exit_jail_unexist($cmd[0]);
	}
	shift @cmd;
	if ( $config{use_jid} ) {
		my @jls=`/usr/sbin/jls`;
		shift @jls;

		foreach (@jls) {
			$_ =~ m/\b(\d+)\s+\S+\s+\b([\w\-\.]*)\s+/g;
			if (length($1)!=0 ) { $jls{$1}=$2; }
		}
		if ( defined($jls{$jid}) ) {
			print "=> $jls{$jid}\n" if $config{show_tips};
			renice_jid($jid);
			exec $jexec_command." $jid ".join(' ', map{"'$_'"} @cmd );
			exit;
		}else{
			exit_not_exec($jid);
		}
	}
}

my @jls=`/usr/sbin/jls`;
shift @jls;
%jls=();

foreach (@jls) {
	$_ =~ m/\b(\d+)\s+\S+\s+\b([\w\-\.]*)\s+/g;
	if (length($1)!=0 ) { $jls{$2}=$1; }
}

fill_list();
my $rcjid=$jid;
if ( !$config{use_jid} ) {
	if ( defined($jid) ) {
		if ( defined($nlist[$jid]) ) {
			print "=> $nlist[$jid]" if $config{show_tips};
			renice_rcjid($jid);
			$jid=defined($jls{$nlist[$jid]})?$jls{$nlist[$jid]}:-1;
			print " $jid\n" if $config{show_tips};
			if ( $jid<0 ) {
				exit_jail_is_stop($nlist[$rcjid]);
			}
			exec $jexec_command." $jid ".join(' ', map{"'$_'"} @cmd );
			exit;
		}else {
			exit_jail_unexist($rcjid);
		}
	}
}
#print $jls{$cmd[0]}."\n";exit;
if ( defined($jls{$cmd[0]}) ) {
	$jid=$jls{$cmd[0]};
	print "=> $jid\n" if $config{show_tips};
	renice_hostname(shift @cmd);
	exec $jexec_command." $jid ".join(' ', map{"'$_'"} @cmd );
	exit;
}

if ( defined($list_name2hostname{$cmd[0]}) ) {
		if ( defined($jls{$list_name2hostname{$cmd[0]}}) ) {
		$jid=$jls{$list_name2hostname{$cmd[0]}};
		my $jname=shift @cmd;
		print "=> $list_name2hostname{$jname} $jid\n" if $config{show_tips};
		renice_name($jname);
		exec $jexec_command." $jid ".join(' ', map{"'$_'"} @cmd );
		exit;
	}else{
		exit_jail_is_stop($list_name2hostname{$cmd[0]});
	}
}

if ( defined($list_hostname2name{$cmd[0]}) ) {
	exit_jail_is_stop($list_hostname2name{$cmd[0]});
}

# only 1 parametr, and it is not a jid or etc.
exit_not_exec("'$cmd[0]'");
